/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package polyliner.domain.impl.facade.scalesnap;

import java.util.ArrayList;
import java.util.List;
import polyliner.domain.exceptions.PointNotFoundException;
import polyliner.domain.impl.point.snap.SnapGrid;
import polyliner.domain.interfaces.FilterCondition;
import polyliner.domain.interfaces.Grid;
import polyliner.domain.interfaces.GridStateChangeListener;
import polyliner.domain.interfaces.Point;
import polyliner.domain.utilities.scale.ScaleGrid;

/**
 *
 * @author dario
 */
public class ScaleSnapGrid implements Grid, GridStateChangeListener {

    private SnapGrid snapGrid;
    private ScaleGrid scaleGrid;
   
    private List<GridStateChangeListener> listeners;
   
    public ScaleSnapGrid(double width, double height,double grid, boolean snap) {
        this.snapGrid = new SnapGrid(width, height,grid,snap);
        this.scaleGrid = new ScaleGrid(snapGrid, 1);
        this.listeners = new ArrayList<GridStateChangeListener>();
        this.scaleGrid.addStateChangeListener(this);
    }
    
    /* Grid Methods */
    public Point addPoint(double x, double y) {
        return this.scaleGrid.addPoint(x, y);
    }

    public Point removePoint(int id) throws PointNotFoundException {
        return this.scaleGrid.removePoint(id);
    }

    public void removeAll() {
        this.scaleGrid.removeAll();
    }

    public Point getPoint(int id) throws PointNotFoundException {
        return this.scaleGrid.getPoint(id);
    }

    public Point[] getPoints() {
        return this.scaleGrid.getPoints();
    }

    public Point[] getPoints(FilterCondition c) {
        return this.scaleGrid.getPoints(c);
    }

    public double getWidth() {
        return this.scaleGrid.getWidth();
    }

    public double getHeight() {
        return this.scaleGrid.getHeight();
    }

    public void addStateChangeListener(GridStateChangeListener listener) {
        if(!this.listeners.contains(listener))
            this.listeners.add(listener);
    }

    public void removeStateChangeListener(GridStateChangeListener listener) {
        if(this.listeners.contains(listener))
            this.listeners.remove(listener);
    }

    /* Scale Methods */
    public void setScale(double factor) {
        this.scaleGrid.setScaleFactor(factor);
    }
    
    public double getScale() {
        return this.scaleGrid.getScaleFactor();
    }
    
    /* Snap Methods */
    public void setSnap(boolean snap) {
        this.snapGrid.setSnap(snap);
    }
    
    public boolean isSnap() {
        return this.snapGrid.isSnap();
    }
    
    public void setGridSize(double size) {
        this.snapGrid.setGridSize(size);
    }
    
    public double getGridSize() {
        return this.snapGrid.getGridSize();
    }
    
    protected void stateChangeUpdate() {
        
        for(GridStateChangeListener l : this.listeners)
            l.stateChanged(this);
    }
    
    protected void addPointUpdate(Point p) {
        
        for(GridStateChangeListener l : this.listeners)
            l.pointAdded(this,p);
        stateChangeUpdate();
    }
    
    protected void removePointUpdate(Point p) {
        
        for(GridStateChangeListener l : this.listeners)
            l.pointRemoved(this,p);
        stateChangeUpdate();
    }
    
    public void pointAdded(Grid grid, Point p) {
        addPointUpdate(p);
    }

    public void pointRemoved(Grid grid, Point p) {
        removePointUpdate(p);
    }

    public void stateChanged(Grid grid) {
        stateChangeUpdate();
    }
}
