/*
 * DefaultPoint.java
 *
 * Created on 17 dicembre 2007, 21.41
 *
 */

package polyliner.domain.impl.point.simple;

import java.util.LinkedHashSet;
import java.util.Set;
import polyliner.domain.interfaces.Point;
import polyliner.domain.interfaces.PointStateChangeListener;

/**
 *
 * @author Dario Lucia
 */
public class DefaultPoint implements Point {
    
    private int id;
    private double x;
    private double y;
    private boolean selected;
    
    private Set<PointStateChangeListener> listeners;
   
    protected DefaultPoint(int id,double x, double y) {
        this.id = id;
        this.x = x;
        this.y = y;
        this.selected = false;
        this.listeners = new LinkedHashSet<PointStateChangeListener>();
    }

    public int getId() {
        return this.id;
    }

    public double getX() {
        return this.x;
    }

    public double getY() {
        return this.y;
    }

    /* TODO: clarify is update() must be called always or only when data effectively change */
    public void setX(double x) {
        this.x = x;
        update();
    }

    public void setY(double y) {
        this.y = y;
        update();
    }

    public void setCoords(double x, double y) {
        this.x = x;
        this.y = y;
        update();
    }
    
    public void move(double x, double y) {
        this.setCoords(this.x + x,this.y + y);
    }

    public void moveX(double x) {
        this.setX(this.x + x);
    }

    public void moveY(double y) {
        this.setY(this.y + y);
    }

    public double getDistance(Point p) {
        return Math.sqrt( Math.pow(p.getX() - this.getX(),2) + Math.pow(p.getY() - this.getY(),2) );
    }

    public void addStateChangeListener(PointStateChangeListener listener) {
        if(!this.listeners.contains(listener))
            this.listeners.add(listener);
    }

    public void removeStateChangeListener(PointStateChangeListener listener) {
        if(this.listeners.contains(listener))
            this.listeners.remove(listener);
    }

    protected void update() {
        for(PointStateChangeListener l : this.listeners)
            l.stateChanged(this);
    }

    @Override
    public boolean equals(Object ob) {
        Point o = (Point) ob;
        return o.getId() == this.getId();
    }
    
    @Override
    public int hashCode() {
        return this.getId();
    }
    
    @Override
    public String toString() {
        return "Point[" + this.getId() + "] (" + this.getX() + ";" + this.getY() + ")";
    }

    public boolean same(Point p) {
        return p.getX()==this.getX() && p.getY()==this.getY();
    }

    public double getDistance(double x, double y) {
        return Math.sqrt( Math.pow(x - this.getX(),2) + Math.pow(y - this.getY(),2) );
    }

    public void select() {
        this.selected = true;
        update();
        
    }

    public void deselect() {
        this.selected = false;
        update();
    }

    public boolean isSelected() {
        return this.selected;
    }

    public double getDistance(Point p1, Point p2) {
        // Calc rect
        if(p1.getX() != p2.getX()) {
            double m = ( p1.getY() - p2.getY() ) / ( p1.getX() - p2.getX() );
            double q = p1.getY() - m * p1.getX();
            return (Math.abs(m*this.getX() - this.getY() + q)) / (Math.sqrt(1 + m*m));
        } else {
            return Math.abs(p1.getX() - this.getX());
        }
    }
    
}
