/*
 * SnapPoint.java
 *
 * Created on 17 dicembre 2007, 23.15
 *
 */

package polyliner.domain.impl.point.snap;

import polyliner.domain.impl.point.simple.DefaultPoint;

/**
 *
 * @author Dario Lucia
 */
public class SnapPoint extends DefaultPoint {
    
    private double gridSize;
    private boolean toSnap;
    
    protected SnapPoint(int id, double x, double y) {
        this(id,x,y,0,false);
    }
    
    protected SnapPoint(int id, double x, double y,double gridSize) {
        this(id,x,y,gridSize,false);
    }
    
    protected SnapPoint(int id, double x, double y, double gridSize,boolean snap) {
        super(id,x,y);
        this.gridSize = gridSize;
        this.toSnap = snap;
        if(toSnap && gridSize != 0) {
            // Refine point
            this.setCoords(this.getX(),this.getY());
        }
    }
    
    public double getGridSize() {
        return this.gridSize;
    }
    
    public boolean isSnap() {
        return this.toSnap;
    }
    
    void setGridSize(double gridSize) {
        this.gridSize = gridSize;
        if(toSnap && gridSize != 0) {
            // Refine point
            this.setCoords(this.getX(),this.getY());
        }
    }
    
    void setSnap(boolean snap) {
        this.toSnap = snap;
        if(toSnap && gridSize != 0) {
            // Refine point
            this.setCoords(this.getX(),this.getY());
        }
    }
    
    /* Overriden methods */
    
    public void setX(double x) {
        if(toSnap && gridSize != 0) {
            double thr = x % gridSize;
            if(thr > gridSize/2) {
                super.setX(x + (gridSize - thr));
            } else {
                super.setX(x - thr);
            }
        } else {
            super.setX(x);
        }
    }

    public void setY(double y) {
        if(toSnap && gridSize != 0) {
            double thr = y % gridSize;
            if(thr > gridSize/2) {
                super.setY(y + (gridSize - thr));
            } else {
                super.setY(y - thr);
            }
        } else {
            super.setY(y);
        }
    }

    public void setCoords(double x, double y) {
        double tX;
        double tY;
        if(toSnap && gridSize != 0) {
            double thr = x % gridSize;
            if(thr > gridSize/2) {
                tX = x + (gridSize - thr);
            } else {
                tX = x - thr;
            }
            
            thr = y % gridSize;
            if(thr > gridSize/2) {
                tY = y + (gridSize - thr);
            } else {
                tY = y - thr;
            }
            
        } else {
            tX = x;
            tY = y;
        }
        
        super.setCoords(tX,tY);
    }
    
    public void move(double x, double y) {
        this.setCoords(this.getX() + x, this.getY() + y);        
    }

    public void moveX(double x) {
        this.setX(this.getX() + x);
    }

    public void moveY(double y) {
        this.setY(this.getY() + y);
    }
}
