/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package polyliner.domain.impl.polyline.simple;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import polyliner.domain.impl.polyline.AbstractPolyline;
import polyliner.domain.interfaces.Grid;
import polyliner.domain.interfaces.Point;

/**
 *
 * @author dario
 */
public class DefaultPolyline extends AbstractPolyline {

    private static final int BOX_MARGIN = 5;
    
    private List<Point> points;
    
    public DefaultPolyline(String name,Grid grid) {
        super(name);
        this.points = new ArrayList<Point>();
        grid.addStateChangeListener(this);
    }
    
    @Override
    public int addPoint(Point p) {
        if(!this.points.isEmpty()) {
            if(this.points.get(this.points.size()-1) != p)
                this.points.add(p);
        } else {
            this.points.add(p);
        }
        
        super.update();
        return this.points.size() - 1;
    }

    @Override
    public int addPoint(Point p, int position) {
        if(position >= 0 && position <= this.points.size()) {
            this.points.add(position, p);
            
            super.update();
            return position;
        } else 
            return -1;
    }

    @Override
    public Point removePoint(Point p) {
        if(this.points.contains(p)) {
            this.points.remove(p); 
            super.update();
            return p;
        } else 
            return null;
        
    }

    @Override
    public Point removePoint(int index) {
        if(index >= 0 && index < this.points.size()) {
            Point p = this.points.remove(index);
            
            super.update();
            return p;
        } else 
            return null;
    }
    
    @Override
    public Point removePointById(int id) {
        for(Iterator<Point> it = this.points.iterator();it.hasNext();) {
            Point p = it.next();
            if(p.getId() == id) { 
                it.remove(); 
                
                super.update();
                return p; 
            }
        }
        return null;
    }

    @Override
    public Point[] getPoints() {
       return this.points.toArray(new Point[this.points.size()]);
    }

    public void pointAdded(Grid grid, Point p) {
        // Nothing to do
    }

    public void pointRemoved(Grid grid, Point p) {
        while(this.points.contains(p)) {
            this.removePoint(p);
            super.update();
        }
    }

    public void stateChanged(Grid grid) {
        //
    }

    public int insertPoint(Point p) {
        int index = -1;
        System.out.println("DefaultPolyline:InsertPoint");
        double currentDistance = -1;
        for(int i = 0;i<this.points.size()-1;++i) {
            System.out.println("DefaultPolyline:Checking " + i);
            if(isInBox(p,this.points.get(i), this.points.get(i+1))) {
                System.out.println("DefaultPolyline:InBox");
                double tempDistance = p.getDistance(this.points.get(i), this.points.get(i+1));
                System.out.println("DefaultPolyline:Distance " + tempDistance);
                if(currentDistance == -1 || currentDistance > tempDistance) {
                    index = i+1;
                    currentDistance = tempDistance;
                }
            }
        }
        if(index != -1) {
            this.addPoint(p, index);
            System.out.println("DefaultPolyline:lastIndex " + index);
        }
        return index;
    }

    private boolean isInBox(Point p, Point p1, Point p2) {
        if(p.getX() > (Math.min(p1.getX(), p2.getX()) - BOX_MARGIN)  &&  p.getX() < (Math.max(p1.getX(), p2.getX()) + BOX_MARGIN) && p.getY() > (Math.min(p1.getY(), p2.getY()) - BOX_MARGIN) && p.getY() < (Math.max(p1.getY(), p2.getY())) + BOX_MARGIN)
            return true;
        else
            return false;
    }

    @Override
    public String toString() {
        return this.getName();
    }
}
