/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package polyliner.domain.utilities.draw;

import java.awt.Graphics;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import polyliner.domain.interfaces.Grid;
import polyliner.domain.interfaces.GridStateChangeListener;
import polyliner.domain.interfaces.Point;

/**
 *
 * @author dario
 */
public class DrawPointManager implements GridStateChangeListener {

    private DrawPointFactory factory;
    
    private Map<Integer,DrawPoint> drawPoints;
    protected Set<DrawPointListener> listeners;
    
  
    public DrawPointManager(Grid grid, DrawPointFactory factory) {
        this.listeners = new LinkedHashSet<DrawPointListener>();
        grid.addStateChangeListener(this);
        this.drawPoints = new TreeMap<Integer, DrawPoint>();
        this.factory = factory;
        for(Point p : grid.getPoints()) {
            this.pointAdded(grid, p);
        }
    }

    public void pointAdded(Grid grid, Point p) {
        DrawPoint point = this.factory.newDrawablePoint(p);
        this.drawPoints.put(p.getId(), point);
        
        update();
    }

    public void pointRemoved(Grid grid, Point p) {
        DrawPoint poi = this.drawPoints.remove(p.getId());
        
        if(poi == null) 
            throw new IllegalStateException("Warning! Deleted point wasn't in drawPoint map");
        else
            update();
    }

    public void stateChanged(Grid grid) {
        
        update();
    }
    
    public void paint(Graphics g) {
        for(DrawPoint p : this.drawPoints.values())
            p.paint(g);
    }
    
    public Point selected(int x, int y) {
        for(DrawPoint p : this.drawPoints.values()) {
            if(p.selected(x, y))
                return p.getPoint();
        }
        return null;
    }
    
    public void addDrawPointListener(DrawPointListener listener) {
        if(!this.listeners.contains(listener))
            this.listeners.add(listener);
    }
    
    public void removeDrawPointListener(DrawPointListener listener) {
        if(this.listeners.contains(listener))
            this.listeners.remove(listener);
    }

    protected void update() {
        
        for(DrawPointListener dpl : this.listeners)
            dpl.drawStateChanged();
    }
}
