/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package polyliner.domain.utilities.draw;

import java.awt.Graphics;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import polyliner.domain.interfaces.Polyline;
import polyliner.domain.interfaces.PolylineStateChangeListener;

/**
 *
 * @author dario
 */
public class DrawPolylineManager implements PolylineStateChangeListener {

    private Map<String,DrawPolyline> polis;
    private DrawPolylineFactory factory;
    protected Set<DrawPolylineListener> listeners;
    
    
    public DrawPolylineManager(DrawPolylineFactory factory) {
        this.polis = new TreeMap<String,DrawPolyline>();     
        this.listeners = new LinkedHashSet<DrawPolylineListener>();
        this.factory = factory;
    }
    
    public void paint(Graphics g) {
        for(DrawPolyline dpl : this.polis.values())
            dpl.paint(g);
    }
    
    public void attach(Polyline p) {
        String name = p.getName();
        if(!this.polis.containsKey(name)) {
            DrawPolyline dpl = this.factory.newDrawablePolyline(p);
            this.polis.put(name, dpl);
            p.addPolylineStateChangeListener(this);
            update();
        }
    }
    
    public void detach(Polyline p) {
        detach(p.getName());
    }
    
    public void detach(String polyName) {
        if(this.polis.containsKey(polyName)) {
            DrawPolyline dpl = this.polis.remove(polyName);
            dpl.getPolyline().removePolylineStateChangeListener(this);
            update();
        }
    }
    
    public void addDrawPolylineListener(DrawPolylineListener listener) {
        if(!this.listeners.contains(listener))
            this.listeners.add(listener);
    }
    
    public void removeDrawPolylineListener(DrawPolylineListener listener) {
        if(this.listeners.contains(listener))
            this.listeners.remove(listener);
    }
    
    protected void update() {
        System.out.println("DrawPolylineManager:update");
        for(DrawPolylineListener dpl : this.listeners)
            dpl.drawStateChanged();
    }

    public void stateChanged(Polyline p) {
        update();
    }
    
}
