/*
 * DrawPoint.java
 *
 * Created on 22 dicembre 2007, 12.54
 *
 */

package polyliner.domain.utilities.draw.impl;

import java.awt.Color;
import java.awt.Graphics;
import polyliner.domain.interfaces.Point;
import polyliner.domain.utilities.draw.*;

/**
 *
 * @author Dario Lucia
 */
public class DefaultDrawPoint extends AbstractDrawPoint {
    
    private static final int FIXED_INNER_RADIUS = 2;
    private static final int FIXED_OUTER_RADIUS = 10;
    
    private boolean selected;
    
    private int innerRadius;
    private int outerRadius;
    
    public DefaultDrawPoint(Point point) {
        this(point,FIXED_INNER_RADIUS,FIXED_OUTER_RADIUS);
    }
    
    /** Creates a new instance of DrawPoint */
    public DefaultDrawPoint(Point point,int innerRadius, int outerRadius) {
        super(point);
        this.setInnerRadius(innerRadius);
        this.setOuterRadius(outerRadius);
        this.selected = false;
    }

    public boolean selected(int x, int y) {
        System.out.println("Distance from " + point.toString() + " to " + x + ";" + y);
        return this.point.getDistance(x,y) <= this.outerRadius;
    }
    
    public void paint(Graphics g) {
        if(!this.point.isSelected()) {
            g.setColor(Color.GREEN);
            g.drawOval((int) Math.round(this.getPoint().getX()) - outerRadius , (int) Math.round(this.getPoint().getY()) - outerRadius ,  
                    outerRadius*2,outerRadius*2);
            g.fillRect((int) Math.round(this.getPoint().getX()) - innerRadius, (int) Math.round(this.getPoint().getY()) - innerRadius,
                    innerRadius*2,innerRadius*2);
        } else {
            g.setColor(Color.RED);
            g.drawOval((int) Math.round(this.getPoint().getX()) - outerRadius , (int) Math.round(this.getPoint().getY()) - outerRadius ,  
                    outerRadius*2,outerRadius*2);
            g.fillRect((int) Math.round(this.getPoint().getX()) - innerRadius, (int) Math.round(this.getPoint().getY()) - innerRadius,
                    innerRadius*2,innerRadius*2);
        }
    }

    public int getInnerRadius() {
        return innerRadius;
    }

    private void setInnerRadius(int innerRadius) {
        this.innerRadius = innerRadius;
    }

    public int getOuterRadius() {
        return outerRadius;
    }

    private void setOuterRadius(int outerRadius) {
        this.outerRadius = outerRadius;
    }

}
