/*
 * ScalePoint.java
 *
 * Created on 22 dicembre 2007, 10.58
 *
 */

package polyliner.domain.utilities.scale;

import java.util.LinkedHashSet;
import java.util.Set;
import polyliner.domain.interfaces.Point;
import polyliner.domain.interfaces.PointStateChangeListener;

/**
 *
 * @author Dario Lucia
 */
public class ScalePoint implements Point,PointStateChangeListener {
    
    private Point basePoint;
    private double scaleFactor;
    
    private Set<PointStateChangeListener> listeners;
    
    /** Creates a new instance of ScalePoint */
    ScalePoint(Point base,double scaleFactor) {
        this.basePoint = base;
        this.scaleFactor = scaleFactor;
        
        this.listeners = new LinkedHashSet<PointStateChangeListener>();
        this.basePoint.addStateChangeListener(this);
    }

    public int getId() {
        return basePoint.getId();
    }

    public double getX() {
        return this.scaleFactor * basePoint.getX();
    }

    public double getY() {
        return this.scaleFactor * basePoint.getY();
    }

    public void setX(double x) {
        this.basePoint.setX(x/scaleFactor);
    }

    public void setY(double y) {
        this.basePoint.setY(y/scaleFactor);
    }

    public void setCoords(double x, double y) {
        
        this.basePoint.setCoords(x/scaleFactor,y/scaleFactor);
    }

    public void move(double x, double y) {
        this.basePoint.move(x/scaleFactor,y/scaleFactor);
    }

    public void moveX(double x) {
        this.basePoint.moveX(x/scaleFactor);
    }

    public void moveY(double y) {
        this.basePoint.moveY(y/scaleFactor);
    }

    public double getDistance(Point p) {
        return Math.sqrt( Math.pow(p.getX() - this.getX(),2) + Math.pow(p.getY() - this.getY(),2) );
    }

    public boolean same(Point p) {
        return this.getX()==p.getX() && this.getY()==p.getY();
    }

    public void addStateChangeListener(PointStateChangeListener listener) {
        if(!this.listeners.contains(listener))
            this.listeners.add(listener);
    }

    public void removeStateChangeListener(PointStateChangeListener listener) {
        if(this.listeners.contains(listener))
            this.listeners.remove(listener);
    }

    void setScaleFactor(double scaleFactor) {
        this.scaleFactor = scaleFactor;
        update();
    }
    
    @Override
    public boolean equals(Object ob) {
        Point o = (Point) ob;
        return o.getId() == this.getId();
    }
    
    @Override
    public int hashCode() {
        return this.getId();
    }
    
    @Override
    public String toString() {
        return "ScalePoint[" + this.getId() + "] (" + this.getX() + ";" + this.getY() + ")";
    }

    public void stateChanged(Point p) {
        update();
    }
    
    private void update() {
        
        for(PointStateChangeListener l : this.listeners)
            l.stateChanged(this);
    }

    public double getDistance(double x, double y) {
        return Math.sqrt( Math.pow(x - this.getX(),2) + Math.pow(y - this.getY(),2) );
    }

    public void select() {
        this.basePoint.select();
    }

    public void deselect() {
        this.basePoint.deselect();
    }

    public boolean isSelected() {
        return this.basePoint.isSelected();
    }

    public double getDistance(Point p1, Point p2) {
        // Calc rect
        if(p1.getX() != p2.getX()) {
            double m = ( p1.getY() - p2.getY() ) / ( p1.getX() - p2.getX() );
            double q = p1.getY() - m * p1.getX();
            return (Math.abs(m*this.getX() - this.getY() + q)) / (Math.sqrt(1 + m*m));
        } else {
            return Math.abs(p1.getX() - this.getX());
        }
    }
    
    public double getXwithoutScaleFactor() {
        return this.basePoint.getX();
    }
    
    public double getYwithoutScaleFactor() {
        return this.basePoint.getY();
    }

}
