/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package polyliner.exporters.impl;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import polyliner.domain.interfaces.Grid;
import polyliner.domain.interfaces.Point;
import polyliner.domain.interfaces.Polyline;
import polyliner.domain.utilities.scale.ScaleGrid;
import polyliner.domain.utilities.scale.ScalePoint;
import polyliner.exporters.Exporter;

/**
 *
 * @author dario
 */
public class DefaultExporter implements Exporter {

    private File fileName;
    
    public DefaultExporter(String fileName) {
        this.fileName = new File(fileName);
    }
    
    public boolean export(Grid grid, Polyline[] polylines) {
        
        try {
            if(!this.fileName.exists())
                this.fileName.createNewFile();
            PrintStream out = new PrintStream(this.fileName);
            out.println("<polyliner>");
            // Export grid
            exportGrid(out,grid);
            // Export polylines
            exportPolys(out,polylines);
            out.println("</polyliner>");
            out.flush();
            out.close();
        } catch (IOException e) {
            return false;
        }
        return true;
    }

    private void exportGrid(PrintStream out, Grid grid) {
        if(grid.getClass().isAssignableFrom(ScaleGrid.class)) {
            ScaleGrid sgrid = (ScaleGrid) grid;
            out.println("\t<grid width=\"" + grid.getWidth()/sgrid.getScaleFactor() + "\" height=\"" + grid.getHeight()/sgrid.getScaleFactor() + "\">" );
        } else {
            out.println("\t<grid width=\"" + grid.getWidth() + "\" height=\"" + grid.getHeight() + "\">" );
        }
        for(Point p : grid.getPoints()) {
            if(p.getClass().isAssignableFrom(ScalePoint.class)) {
                 ScalePoint pt = ((ScalePoint) p);
                 out.println("\t\t<point id=\"" + pt.getId() + "\" x=\"" + pt.getXwithoutScaleFactor() + "\" y=\"" + pt.getYwithoutScaleFactor() + "\" />");
            }
            else {
                out.println("\t\t<point id=\"" + p.getId() + "\" x=\"" + p.getX() + "\" y=\"" + p.getY() + "\" />");
            }
        }
        out.println("\t</grid>");
    }

    private void exportPolys(PrintStream out, Polyline[] polylines) {
        out.println("\t<polylines>");
        for(Polyline p : polylines) {
            out.println("\t\t<polyline name=\"" + p.getName() + "\">");
            for(Point pt : p.getPoints()) {
                out.println("\t\t\t<point idref=\"" + pt.getId() + "\" />");
            }
            out.println("\t\t</polyline>");
        }
        out.println("\t</polylines>");
    }

}
