package polyliner.exporters.impl;


import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import polyliner.domain.interfaces.Grid;
import polyliner.domain.interfaces.Point;
import polyliner.domain.interfaces.Polyline;
import polyliner.domain.utilities.scale.ScalePoint;
import polyliner.exporters.Exporter;

/**
 *
 * @author dario
 */
public class PlasmExporter implements Exporter {
    
    private File fileName;
    
    public PlasmExporter(String fileName) {
        this.fileName = new File(fileName);
    }
    
    public boolean export(Grid grid, Polyline[] polylines) {
        try {
            if(!this.fileName.exists())
                this.fileName.createNewFile();
            PrintStream out = new PrintStream(this.fileName);
            out.println("(plasm \"");
            out.println();
            for(Polyline p : polylines)
                exportPoly(out,p);
            out.println();
            out.println("\")");
            out.flush();
            out.close();
        } catch (IOException e) {
            return false;
        }
        return true;
    }

    private void exportPoly(PrintStream out, Polyline p) {
        out.println("DEF " + p.getName() + " = POLYLINE:<");
        boolean coma = true;
        for(Point pt : p.getPoints()) {
            if(coma) {
                coma = false;
            } else {
                out.println(",");
            }
            if(pt.getClass().isAssignableFrom(ScalePoint.class)) {
                 ScalePoint ptt = ((ScalePoint) pt);
                 out.print("\t<" + ptt.getXwithoutScaleFactor() + "," + ptt.getYwithoutScaleFactor() + ">");
            }
            else {
                 out.print("\t<" + pt.getX() + "," + pt.getY() + ">");
            }
        }
        out.println();
        out.println(">;");
    }

}
