package polyliner.importers.impl;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import polyliner.domain.exceptions.PointNotFoundException;
import polyliner.domain.impl.facade.scalesnap.ScaleSnapGrid;
import polyliner.domain.impl.polyline.simple.DefaultPolyline;
import polyliner.domain.interfaces.Grid;
import polyliner.domain.interfaces.Point;
import polyliner.domain.interfaces.Polyline;
import polyliner.importers.Importer;

/**
 *
 * @author dario
 */
public class DefaultScaleSnapImporter implements Importer {

    private ScaleSnapGrid grid;
    private List<Polyline> polylines;
    
    public DefaultScaleSnapImporter() {
        this.grid = null;
        this.polylines = new LinkedList<Polyline>();
    }

    public void load(String filename) {
        this.polylines.clear();
        this.grid = null;
        
        Document doc = parseXmlFile(filename);
        loadFromDoc(doc);
        System.out.println("Doc loaded ");
    }

    public Grid getGrid() {
        return this.grid;
    }

    public Polyline[] getPolylines() {
        return this.polylines.toArray(new Polyline[polylines.size()]);
    }

    protected void loadFromDoc(Document doc) {
        // Load grid
        System.out.println("Doc loaded 2");
        NodeList gridNodes = doc.getElementsByTagName("grid");
        System.out.println("Doc loaded 3");
        if(gridNodes.getLength() != 1)
            throw new IllegalStateException("More grids defined in file");
        System.out.println("Doc loaded 4");
        Element gridNode = (Element) gridNodes.item(0);
        System.out.println("Doc loaded 5");
        double w,h;
        w = Double.parseDouble(gridNode.getAttribute("width"));
        h = Double.parseDouble(gridNode.getAttribute("height"));
        this.grid = new ScaleSnapGrid(w,h,1,false);
        
        int nextId = 0;
        // Load points
        NodeList pointList = gridNode.getElementsByTagName("point");
        System.out.println("Doc loaded 6");
        for(int i = 0;i<pointList.getLength();++i) {
            Element pointNode = (Element) pointList.item(i);
            System.out.println("Doc loaded 6a");
            int id;
            double x,y;
            id = Integer.parseInt(pointNode.getAttribute("id"));
            x = Double.parseDouble(pointNode.getAttribute("x"));
            y = Double.parseDouble(pointNode.getAttribute("y"));
            System.out.println("Doc loaded 6b");
            while(nextId < id) {
                try {
                    Point p = grid.addPoint(0, 0);
                    grid.removePoint(p.getId());
                    ++nextId;
                } catch (PointNotFoundException ex) {
                    Logger.getLogger(DefaultScaleSnapImporter.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            System.out.println("Doc loaded 6c");
            Point dd = this.grid.addPoint(x, y);
            ++nextId;
            System.out.println("Point with id: " + dd.getId());
        }
        System.out.println("Doc loaded 7");
        // Load polylines
        NodeList poliNodes = doc.getElementsByTagName("polyline");
        System.out.println("Doc loaded 8");
        for(int i = 0;i<poliNodes.getLength();++i) {
            Element poliNode = (Element) poliNodes.item(i);
            String name = poliNode.getAttribute("name");
            Polyline pol = new DefaultPolyline(name, grid);
            NodeList ptList = poliNode.getElementsByTagName("point");
            for(int j = 0;j<ptList.getLength();++j) {
                try {
                    Element ptNode = (Element) ptList.item(j);
                    int idref = Integer.parseInt(ptNode.getAttribute("idref"));
                    pol.addPoint(this.grid.getPoint(idref));
                } catch (PointNotFoundException ex) {
                    Logger.getLogger(DefaultScaleSnapImporter.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            this.polylines.add(pol);
        }
    }
    
    protected Document parseXmlFile(String filename) {
            try {
                // Create a builder factory
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                factory.setValidating(false);
    
                // Create the builder and parse the file
                Document doc = factory.newDocumentBuilder().parse(new File(filename));
                return doc;
            } catch (SAXException e) {
                // A parsing error occurred; the xml input is not valid
            } catch (ParserConfigurationException e) {
            } catch (IOException e) {
            }
            return null;
    }
}
